
//*
//* tiatab.c -- tables needed for z26 TIA emulator
//*

// z26 is Copyright 1997-2019 by John Saeger and contributors.  
// contributors.  z26 is released subject to the terms and conditions of the 
// GNU General Public License Version 2 (GPL).  z26 comes with no warranty.
// Please see COPYING.TXT for details.

// 09-07-02 -- 32-bit
// 20190410 -- C

// playfield counter to playfield bit table

//          PF2           |         PF1           |         PF0           |
//  7  6  5  4  3  2  1  0  7  6  5  4  3  2  1  0  7  6  5  4  3  2  1  0
// 23 22 21 20 19 18 17 16 15 14 13 12 11 10  9  8  7  6  5  4  3  2  1  0
//                                  ECX                                   |

dd PFClockToBitReversed[] = {
    1 << 4,     // 0     forward
    1 << 5,     // 1
    1 << 6,     // 2
    1 << 7,     // 3
    1 << 15,    // 4
    1 << 14,    // 5
    1 << 13,    // 6
    1 << 12,    // 7
    1 << 11,    // 8
    1 << 10,    // 9
    1 << 9,     // 10
    1 << 8,     // 11
    1 << 16,    // 12
    1 << 17,    // 13
    1 << 18,    // 14
    1 << 19,    // 15
    1 << 20,    // 16
    1 << 21,    // 17
    1 << 22,    // 18
    1 << 23,    // 19
 
    1 << 23,    // 20    reversed
    1 << 22,    // 21
    1 << 21,    // 22
    1 << 20,    // 23
    1 << 19,    // 24
    1 << 18,    // 25
    1 << 17,    // 26
    1 << 16,    // 27
    1 << 8,     // 28
    1 << 9,     // 29
    1 << 10,    // 30
    1 << 11,    // 31
    1 << 12,    // 32
    1 << 13,    // 33
    1 << 14,    // 34
    1 << 15,    // 35
    1 << 7,     // 36
    1 << 6,     // 37
    1 << 5,     // 38
    1 << 4      // 39
};

dd PFClockToBitForward[] = {
    1 << 4,     // 0     forward
    1 << 5,     // 1
    1 << 6,     // 2
    1 << 7,     // 3
    1 << 15,    // 4
    1 << 14,    // 5
    1 << 13,    // 6
    1 << 12,    // 7
    1 << 11,    // 8
    1 << 10,    // 9
    1 << 9,     // 10
    1 << 8,     // 11
    1 << 16,    // 12
    1 << 17,    // 13
    1 << 18,    // 14
    1 << 19,    // 15
    1 << 20,    // 16
    1 << 21,    // 17
    1 << 22,    // 18
    1 << 23,    // 19
 
    1 << 4,     // 0     forward
    1 << 5,     // 1
    1 << 6,     // 2
    1 << 7,     // 3
    1 << 15,    // 4
    1 << 14,    // 5
    1 << 13,    // 6
    1 << 12,    // 7
    1 << 11,    // 8
    1 << 10,    // 9
    1 << 9,     // 10
    1 << 8,     // 11
    1 << 16,    // 12
    1 << 17,    // 13
    1 << 18,    // 14
    1 << 19,    // 15
    1 << 20,    // 16
    1 << 21,    // 17
    1 << 22,    // 18
    1 << 23     // 19
};


//
// pixel to collision word
//
dw TIAColTab[] = {
    0,          // 0
    0,          // 0
    0,          // 0
    8192,       // 2000
    0,          // 0
    128,        // 80
    64,         // 40
    8384,       // 20c0
    0,          // 0
    2048,       // 800
    1024,       // 400
    11264,      // 2c00
    4,          // 4
    2180,       // 884
    1092,       // 444
    11460,      // 2cc4
    0,          // 0
    32,         // 20
    16,         // 10
    8240,       // 2030
    32768,      // 8000
    32928,      // 80a0
    32848,      // 8050
    41200,      // a0f0
    8,          // 8
    2088,       // 828
    1048,       // 418
    11320,      // 2c38
    32780,      // 800c
    34988,      // 88ac
    33884,      // 845c
    44284,      // acfc
    0,          // 0
    512,        // 200
    256,        // 100
    8960,       // 2300
    2,          // 2
    642,        // 282
    322,        // 142
    9154,       // 23c2
    16384,      // 4000
    18944,      // 4a00
    17664,      // 4500
    28416,      // 6f00
    16390,      // 4006
    19078,      // 4a86
    17734,      // 4546
    28614,      // 6fc6
    1,          // 1
    545,        // 221
    273,        // 111
    9009,       // 2331
    32771,      // 8003
    33443,      // 82a3
    33107,      // 8153
    41971,      // a3f3
    16393,      // 4009
    18985,      // 4a29
    17689,      // 4519
    28473,      // 6f39
    49167,      // c00f
    51887,      // caaf
    50527,      // c55f
    61439       // efff
};

db TIADisplayToColour[] = {
    0,1,4,4,2,2,2,2,2,2,2,2,2,2,2,2,
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
    3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3
};

db TIADisplayToColour2[] = {
    0,1,4,1,2,1,4,1,2,1,4,1,2,1,4,1,
    3,1,4,1,3,1,4,1,3,1,4,1,3,1,4,1,
    3,1,4,1,3,1,4,1,3,1,4,1,3,1,4,1,
    3,1,4,1,3,1,4,1,3,1,4,1,3,1,4,1
};

dd* PF_Table = PFClockToBitForward;         // current pointer to playfield mask bits
db* PixelColorTable = TIADisplayToColour;   // pixel to color translation table

// ;*
// ;* sprite definitions
// ;*
// ;* The top bit of the delay byte is the triple flag.
// ;* This means subsequent RESPx outputs the leading sprite.
// ;*
// ;* Note:  There is an entry just before the first entry of a single-width 
// ;*        player sprite that isn't there, that should have the triple flag
// ;*        set.  That is there should be four entries with the triple flag set.
// ;* 

#define LeadingPlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      10, \
    0,      9, \
    0x80,   8, \
    0x40,   7, \
    0x20,   6, \
    0x10,   5, \
    0x8,    4, \
    0x4,    3, \
    0x2,    2, \
    0x1,    1, \
    0,      0, \

#define BlankPlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      10, \
    0,      9, \
    0,      8, \
    0,      7, \
    0,      6, \
    0,      5, \
    0,      4, \
    0,      3, \
    0,      2, \
    0,      1, \
    0,      0, \

#define TrailingPlayer \
    0,      0, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      10, \
    0,      9, \
    0x80,   8, \
    0x40,   7, \
    0x20,   6, \
    0x10,   5, \
    0x8,    4, \
    0x4,    3, \
    0x2,    2, \
    0x1,    1, \
    0,      0, \

#define LeadingMissile \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0, \
    0x80,   8, \
    0x40,   7, \
    0x20,   6, \
    0x10,   5, \
    0x8,    4, \
    0x4,    3, \
    0x2,    2, \
    0x1,    1, \
    0,      0, \

#define BlankMissile \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0, \
    0,      8, \
    0,      7, \
    0,      6, \
    0,      5, \
    0,      4, \
    0,      3, \
    0,      2, \
    0,      1, \
    0,      0, \

#define TrailingMissile \
    0,      0, \
    0,      0, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0, \
    0x80,   8, \
    0x40,   7, \
    0x20,   6, \
    0x10,   5, \
    0x8,    4, \
    0x4,    3, \
    0x2,    2, \
    0x1,    1, \
    0,      0, \

#define MediumGap \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \

#define WideGap \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, \

// ;*
// ;* Medium and wide players have an extra leading clock.
// ;* They have the extra triple flag too.
// ;*

#define MediumPlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      18, \
    0,      17, \
    0x80,   16, \
    0x80,   15, \
    0x40,   14, \
    0x40,   13, \
    0x20,   12, \
    0x20,   11, \
    0x10,   10, \
    0x10,   9, \
    0x8,    8, \
    0x8,    7, \
    0x4,    6, \
    0x4,    5, \
    0x2,    4, \
    0x2,    3, \
    0x1,    2, \
    0x1,    1, \
    0,      0, \

#define MediumBlankPlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      18, \
    0,      17, \
    0,      16, \
    0,      15, \
    0,      14, \
    0,      13, \
    0,      12, \
    0,      11, \
    0,      10, \
    0,      9, \
    0,      8, \
    0,      7, \
    0,      6, \
    0,      5, \
    0,      4, \
    0,      3, \
    0,      2, \
    0,      1, \
    0,      0, \

#define WidePlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      34, \
    0,      33, \
    0x80,   32, \
    0x80,   31, \
    0x80,   30, \
    0x80,   29, \
    0x40,   28, \
    0x40,   27, \
    0x40,   26, \
    0x40,   25, \
    0x20,   24, \
    0x20,   23, \
    0x20,   22, \
    0x20,   21, \
    0x10,   20, \
    0x10,   19, \
    0x10,   18, \
    0x10,   17, \
    0x8,    16, \
    0x8,    15, \
    0x8,    14, \
    0x8,    13, \
    0x4,    12, \
    0x4,    11, \
    0x4,    10, \
    0x4,    9, \
    0x2,    8, \
    0x2,    7, \
    0x2,    6, \
    0x2,    5, \
    0x1,    4, \
    0x1,    3, \
    0x1,    2, \
    0x1,    1, \
    0,      0, \

#define WideBlankPlayer \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      0x80, \
    0,      34, \
    0,      33, \
    0,      32, \
    0,      31, \
    0,      30, \
    0,      29, \
    0,      28, \
    0,      27, \
    0,      26, \
    0,      25, \
    0,      24, \
    0,      23, \
    0,      22, \
    0,      21, \
    0,      20, \
    0,      19, \
    0,      18, \
    0,      17, \
    0,      16, \
    0,      15, \
    0,      14, \
    0,      13, \
    0,      12, \
    0,      11, \
    0,      10, \
    0,      9, \
    0,      8, \
    0,      7, \
    0,      6, \
    0,      5, \
    0,      4, \
    0,      3, \
    0,      2, \
    0,      1, \
    0,      0, \

// ;*
// ;* sprite size calculation macro
// ;*

// %macro SpriteSize 1
// %1_Size equ ($-%1)/2-1 
// %endmacro

#define sizeofsprite($1) (sizeof($1)/2 - 1)

//
// default sprite definitions
//

// P_0 -- Player NUSIZ = 0

db P_0[] = { 
    LeadingPlayer 
};

// M_0 -- Missile NUSIZ = 0

db M_0[] = { 
    LeadingMissile
};

// P_1 -- Player NUSIZ = 1

db P_1[] = {
    LeadingPlayer
    TrailingPlayer
};

// M_1 -- Missile NUSIZ = 1

db M_1[] = {
    LeadingMissile
    TrailingMissile
};

// P_2 -- Player NUSIZ = 2

db P_2[] = {
    LeadingPlayer
    MediumGap
    TrailingPlayer
};

// M_2 -- Missile NUSIZ = 2

db M_2[] = {
    LeadingMissile
    MediumGap
    TrailingMissile
};

// P_3 -- Player NUSIZ = 3

db P_3[] = {
    LeadingPlayer
    TrailingPlayer
    TrailingPlayer
};

// M_3 -- Missile NUSIZ = 3

db M_3[] = {
    LeadingMissile
    TrailingMissile
    TrailingMissile
};

// P_4 -- Player NUSIZ = 4

db P_4[] = {
    LeadingPlayer
    WideGap
    TrailingPlayer
};

// M_4 -- Missile NUSIZ = 4

db M_4[] = {
    LeadingMissile
    WideGap
    TrailingMissile
};

// P_5 -- Player NUSIZ = 5

db P_5[] = {
    MediumPlayer
};

// M_5 -- Missile NUSIZ = 5

db M_5[] = { 
    LeadingMissile
};

// P_6 -- Player NUSIZ = 6

db P_6[] = {
    LeadingPlayer
    MediumGap
    TrailingPlayer
    MediumGap
    TrailingPlayer
};

// M_6 -- Missile NUSIZ = 6

db M_6[] = {
    LeadingMissile
    MediumGap
    TrailingMissile
    MediumGap
    TrailingMissile
};

// P_7 -- Player NUSIZ = 7

db P_7[] = {
    WidePlayer
};

// M_7 -- Missile NUSIZ = 7

db M_7[] = { 
    LeadingMissile
};


//
// multi sprite definitions
//

// MP_0 -- Player NUSIZ = 0

db MP_0[] = {
    BlankPlayer
};

// MM_0 -- Missile NUSIZ = 0

db MM_0[] = {
    BlankMissile
};

// MP_1 -- Player NUSIZ = 1

db MP_1[] = {
    BlankPlayer
    TrailingPlayer
};

// MM_1 -- Missile NUSIZ = 1

db MM_1[] = {
    BlankMissile
    TrailingMissile
};
 
// MP_2 -- Player NUSIZ = 2

db MP_2[] = {
    BlankPlayer
    MediumGap
    TrailingPlayer
};

// MM_2 -- Missile NUSIZ = 2

db MM_2[] = {
    BlankMissile
    MediumGap
    TrailingMissile
};
 
// MP_3 -- Player NUSIZ = 3

db MP_3[] = {
    BlankPlayer
    TrailingPlayer
    TrailingPlayer
};
 
// MM_3 -- Missile NUSIZ = 3

db MM_3[] = {
    BlankMissile
    TrailingMissile
    TrailingMissile
};
 
// MP_4 -- Player NUSIZ = 4

db MP_4[] = {
    BlankPlayer
    WideGap
    TrailingPlayer
};

// MM_4 -- Missile NUSIZ = 4

db MM_4[] = {
    BlankMissile
    WideGap
    TrailingMissile
};

// MP_5 -- Player NUSIZ = 5

db MP_5[] = {
    MediumBlankPlayer
};

// MM_5 -- Missile NUSIZ = 5

db MM_5[] = {
    BlankMissile
};

// MP_6 -- Player NUSIZ = 6

db MP_6[] = {
    BlankPlayer
    MediumGap
    TrailingPlayer
    MediumGap
    TrailingPlayer
};

// MM_6 - Missile NUSIZ = 6

db MM_6[] = {
    BlankMissile
    MediumGap
    TrailingMissile
    MediumGap
    TrailingMissile
};
 
// MP_7 -- Player NUSIZ = 7

db MP_7[] = {
    WideBlankPlayer
};

// MM_7 -- Missile NUSIZ = 7

db MM_7[] = {
    BlankMissile
};

//
// regular sprite tables
//
 
db* P0_RegularTable[] = {
    P_0, P_1, P_2, P_3, P_4, P_5, P_6, P_7
};

db** P1_RegularTable = P0_RegularTable;

db P0_SizeTable[] = {
    sizeofsprite(P_0),
    sizeofsprite(P_1),
    sizeofsprite(P_2),
    sizeofsprite(P_3),
    sizeofsprite(P_4),
    sizeofsprite(P_5),
    sizeofsprite(P_6),
    sizeofsprite(P_7),
};

db* P1_SizeTable = P0_SizeTable;

db* M0_RegularTable[] = {
    M_0, M_1, M_2, M_3, M_4, M_5, M_6, M_7
};

db** M1_RegularTable = M0_RegularTable;

db M0_SizeTable[] = {
    sizeofsprite(M_0),
    sizeofsprite(M_1),
    sizeofsprite(M_2),
    sizeofsprite(M_3),
    sizeofsprite(M_4),
    sizeofsprite(M_5),
    sizeofsprite(M_6),
    sizeofsprite(M_7),
};

db* M1_SizeTable = M0_SizeTable;

//
// multiple sprite tables
//
// note: originally there were no size tables for these
//       maybe it's a good idea to use sizes that match the tables...
//

db* P0_MultipleTable[] = {
    MP_0, MP_1, MP_2, MP_3, MP_4, MP_5, MP_6, MP_7
};

db** P1_MultipleTable = P0_MultipleTable;

db P0_MultipleSizeTable[] = {
    sizeofsprite(MP_0),
    sizeofsprite(MP_1),
    sizeofsprite(MP_2),
    sizeofsprite(MP_3),
    sizeofsprite(MP_4),
    sizeofsprite(MP_5),
    sizeofsprite(MP_6),
    sizeofsprite(MP_7),
};

db* P1_MultipleSizeTable = P0_MultipleSizeTable;

db* M0_MultipleTable[] = {
    MM_0, MM_1, MM_2, MM_3, MM_4, MM_6, MM_6, MM_7
};

db** M1_MultipleTable = M0_MultipleTable;

db M0_MultipleSizeTable[] = {
    sizeofsprite(MM_0),
    sizeofsprite(MM_1),
    sizeofsprite(MM_2),
    sizeofsprite(MM_3),
    sizeofsprite(MM_4),
    sizeofsprite(MM_5),
    sizeofsprite(MM_6),
    sizeofsprite(MM_7),
};

db* M1_MultipleSizeTable = M0_MultipleSizeTable;


db BL_Sprite[] = {LeadingMissile};
db M0_Sprite[] = {LeadingMissile};
db M1_Sprite[] = {LeadingMissile};
db P0_Sprite[] = {LeadingPlayer};
db P1_Sprite[] = {LeadingPlayer};


#define M4 0        // I'm not actually sure if this should be zero or -4.
int DeepHMOVE[] = {
/* 0 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 5, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 1 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 8,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 7, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 5, 5, 5, 5, 5,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/*  2  */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 8, 8,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 7, 7, 7,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 3 */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0,10,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 9, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 8, 8, 8,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 7, 7, 7, 7, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 6, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 4 */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0,11,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0,10,10,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 9, 9, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 8, 8, 8, 8, 8,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 7, 7, 7, 7, 7, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 6, 6, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 5 */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0,12,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0,11,11,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0,10,10,10,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 9, 9, 9, 9, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 8, 8, 8, 8, 8, 8,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 7, 7, 7, 7, 7, 7, 7,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 6, 6, 6, 6, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 6 */
 0,0,0,0,0,0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0,13,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0,12,12,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0,11,11,11,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0,10,10,10,10,10,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 9, 9, 9, 9, 9, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 8, 8, 8, 8, 8, 8, 8,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 7, 7, 7, 7, 7, 7, 7, 7, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 7 */
 0,0,0,0,0,0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0,14,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0,13,13,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0,12,12,12,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0,11,11,11,11,11,99,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0,10,10,10,10,10,10,99,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 9, 9, 9, 9, 9, 9, 9,99,99,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 8, 8, 8, 8, 8, 8, 8, 8, 8,99,99,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0,  /* f */
/* 8 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* 9 */
 0,0,0,0,0,0,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-10,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-11,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-12,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-13,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-14,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-14,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* a */
 0,0,0,0,0,0,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-10,-10,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-11,-11,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-12,-12,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-13,-13,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-13,-13,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* b */
 0,0,0,0,0,0,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-9,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-10,-10,-10,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-11,-11,-11,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-12,-12,-12,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-12,-12,-12,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 2,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* c */
 0,0,0,0,0,0,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-8,-8,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-9,-9,-9,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-10,-10,-10,-10,-10,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-11,-11,-11,-11,-11,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-11,-11,-11,-11,-11,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 1, 1, 1,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* d */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-8,-8,-8,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-9,-9,-9,-9,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-10,-10,-10,-10,-10,-10,0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-10,-10,-10,-10,-10,-10,0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* e */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-8,-8,-8,-8,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-9,-9,-9,-9,-9,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-9,-9,-9,-9,-9,-9,-9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 5,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 4, 4,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 2, 2, 2, 2, 2,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
/* f */
 0,0,0,0,0,0,-1,-1,-1,-1,-1,-1,-1,-1,-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 0 */
 0,0,0,0,0,0,-2,-2,-2,-2,-2,-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 1 */
 0,0,0,0,0,0,-3,-3,-3,-3,-3,-3,-3,-3,-3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 2 */
 0,0,0,0,0,0,-4,-4,-4,-4,-4,-4,-4,-4,-4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 3 */
 0,0,0,0,0,0,-5,-5,-5,-5,-5,-5,-5,-5,-5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 4 */
 0,0,0,0,0,0,-6,-6,-6,-6,-6,-6,-6,-6,-6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 5 */
 0,0,0,0,0,0,-7,-7,-7,-7,-7,-7,-7,-7,-7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 6 */
 0,0,0,0,0,0,-8,-8,-8,-8,-8,-8,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 7 */
 0,0,0,0,0,0,-8,-8,-8,-8,-8,-8,-8,-8,-8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 8 */
 0,0,0,0,0,0, 6,99,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* 9 */
 0,0,0,0,0,0, 5, 5,99,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* a */
 0,0,0,0,0,0, 4, 4, 4,99,99,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* b */
 0,0,0,0,0,0, 3, 3, 3, 3, 3,99,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* c */
 0,0,0,0,0,0, 2, 2, 2, 2, 2, 2,99,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* d */
 0,0,0,0,0,0, 1, 1, 1, 1, 1, 1, 1,99,99,M4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* e */
 0,0,0,0,0,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  /* f */
};

dd WeirdRespCorrection[] = {
    0,  0,  0,  0,  0,  0,  1,  2,  2,  3,
    4,  5,  5,  6,  7,  8,  8,  9, 10, 11,
   11, 12, 13, 14, 14, 15
};

db PFDelay[] = { 4, 3, 2, 5 };     // ;  delays for writes to PF registers

db BallSize[] = {
    0x80, // 10000000b
    0xc0, // 11000000b
    0xf0, // 11110000b
    0xff, // 11111111b
};

db TIAReflect8[] = {
    0,128,64,192,32,160,96,224,16,144,80,208,48,176,112,240,
    8,136,72,200,40,168,104,232,24,152,88,216,56,184,120,248,
    4,132,68,196,36,164,100,228,20,148,84,212,52,180,116,244,
    12,140,76,204,44,172,108,236,28,156,92,220,60,188,124,252,
    2,130,66,194,34,162,98,226,18,146,82,210,50,178,114,242,
    10,138,74,202,42,170,106,234,26,154,90,218,58,186,122,250,
    6,134,70,198,38,166,102,230,22,150,86,214,54,182,118,246,
    14,142,78,206,46,174,110,238,30,158,94,222,62,190,126,254,
    1,129,65,193,33,161,97,225,17,145,81,209,49,177,113,241,
    9,137,73,201,41,169,105,233,25,153,89,217,57,185,121,249,
    5,133,69,197,37,165,101,229,21,149,85,213,53,181,117,245,
    13,141,77,205,45,173,109,237,29,157,93,221,61,189,125,253,
    3,131,67,195,35,163,99,227,19,147,83,211,51,179,115,243,
    11,139,75,203,43,171,107,235,27,155,91,219,59,187,123,251,
    7,135,71,199,39,167,103,231,23,151,87,215,55,183,119,247,
    15,143,79,207,47,175,111,239,31,159,95,223,63,191,127,255
};

dd MissileOffset[] = { 5, 5, 5, 5, 5, 8, 5, 12 };

dd MaxMotion[] = {
    7,  7,  7,  7,  6,  5,  5,  4,  3,  2,
    2,  1,  0, -1, -1, -2, -3, -4, -4, -5,
    -6
};

dd HiTable[] = { 
    14, 13, 12, 12, 11, 10, 9, 9, 8, 7, 6, 6, 5, 4, 3, 3, 2, 1, 0, 0 
};
